/*
 *
 * mod_suid functionality tester
 *
 * (c) 2003 Igmar Palsenberg
 * JDI Media Solutions
 *
*/
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/utsname.h>

#define CHTOUSER1	1
#define CHTOUSER2	2

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE 1
#endif

static int change_to_root(void);
static int change_to_user_group(uid_t uid, gid_t gid);
static void print_uid_info(void);
static int check_uid_info(uid_t ruid, uid_t euid, uid_t suid);

int main(int argc, char ** argv)
{
	struct utsname un;

	if (getuid()) {
		printf("Sorry, you need to run this as root\n");
		return 1;
	}

	if (uname(&un) == -1)
		printf("uname() failed : %s\n", strerror(errno));
	else 
		printf("Sys: %s Rel: %s Ver: %s Machine: %s\n", un.sysname, un.release, un.version, un.machine);

	/* Test 1 : Change to uid 1, gid 1 */
	printf("\nTest 1\n");
	if (change_to_user_group(1, 1) == FALSE)
		printf("Test 1 failed, send output to author\n");
	if (check_uid_info(1, 1, 0) == FALSE)
		printf("Test 1 check failed, send output to author\n");
	print_uid_info();

	/* Test 2 : Change to uid 2, gid 2. This should fail */
	printf("\nTest 2 (should fail)\n");
	if (change_to_user_group(2, 2) == TRUE)
		printf("Test 2 failed, send output to author\n");
	/* Yes, this is correct */
	if (check_uid_info(1, 1, 0) == FALSE)
		printf("Test 2 check failed, send output to author\n");
	print_uid_info();
	
	/* Test 3 : Change back to root */
	printf("\nTest 3\n");
	if (change_to_root() == FALSE)
		printf("Test 3 failed, send output to author\n");
	if (check_uid_info(0, 0, 0) == FALSE)
		printf("Test 3 check failed, send output to author\n");
	print_uid_info(); 

	/* Test 4 : Change to uid 2, gid 2 */
	printf("\nTest 4\n");
	if (change_to_user_group(2, 2) == FALSE)
		printf("Test 4 failed, send output to author\n");
	if (check_uid_info(2, 2, 0) == FALSE)
		printf("Test 4 check failed, send output to author\n");
	print_uid_info();

	/* Test 5 : Change back to root */
	printf("\nTest 5\n");
	if (change_to_root() == FALSE)
		printf("Test 5 failed, send output to author\n");
	if (check_uid_info(0, 0, 0) == FALSE)
		printf("Test 5 check failed, send output to author\n");
	print_uid_info(); 

	return 0;	
}

static int change_to_root(void)
{
	if (setreuid(-1, 0) == -1) {
		printf("change_to_root : setreuid(-1,0) failed : %s\n", strerror(errno));
		return FALSE;
	}
	if (setreuid(0,-1) == -1) {
		printf("change_to_root : setreuid(0,-1) failed : %s\n", strerror(errno));
		return FALSE;
	}
	if (setregid(-1, 0) == -1) {
		printf("change_to_root : setregid(-1,0) failed : %s\n", strerror(errno));
		return FALSE;
	}
	if (setregid(0,-1) == -1) {
		printf("change_to_root : setregid(0,-1) failed : %s\n", strerror(errno));
		return FALSE;
	}
	return TRUE;	
}

static int change_to_user_group(uid_t uid, gid_t gid)
{
	if (setresgid(gid, gid, 0) == -1) {
		printf("change_to_user_group : setresgid(%d, %d, 0) failed : %s\n", gid, gid, strerror(errno));
		return FALSE;
	}
	if (setresuid(uid, uid, 0) == -1) {
		printf("change_to_user_group : setresuid(%d, %d, 0) failed : %s\n", uid, uid, strerror(errno));
		return FALSE;
	}
	return TRUE;
}

static void print_uid_info(void)
{
	uid_t ruid, euid, suid;
	gid_t rgid, egid, sgid;
	
	getresuid(&ruid, &euid, &suid);
	getresgid(&rgid, &egid, &sgid);

	printf("ruid %d euid %d suid %d\n", ruid, euid, suid);
	printf("rgid %d egid %d sgid %d\n", rgid, egid, sgid);
}

static int check_uid_info(uid_t ruid, uid_t euid, uid_t suid)
{
	uid_t xruid, xeuid, xsuid;
	
	getresuid(&xruid, &xeuid, &xsuid);
	if (xruid == ruid && xeuid == euid && xsuid == suid)
		return TRUE;

	return FALSE;
}
